/*
Encoder.s

Copyright 1998-2000 Larry Barello

Author: Larry Barello
        larry@barello.net
*/
#define	__SFR_OFFSET 0       	/* GCC 3.2 needs this for direct low I/O access */
#include 	"sig-avr.h"
#include        "AvrX.inc"
#include        "hardware.h"
;
; New Encoder
;
; Interrupt driven off of INT0 and INT1, GIMSK and MCUCR
; initial values defined in "hardware.h" and set in main()
;
        _MODULE(encoder)
        _DATASECTION
        _GLOBAL(RightEncoder, 1)
        _GLOBAL(LeftEncoder, 1)
;+
; INT0_Interrupt
;
; Fields the A line for an encoder.  Increments/Decrements a
; register counter per the state of the B line.  Flips sense of
; external interrupt in preparation for the next state transistion.
;
; Because we dont interrupt on the B line, this only does
; 1/2 quadurature decoding.
;
; ~40 cycles/interrupt (including call and return)
;-
        _CODESECTION
        _FUNCTION(SIG_INTERRUPT0)

SIG_INTERRUPT0:
        BeginInterrupt
        push    R31
        push    R30
        push    R29
        in      R31, SREG
        in      R30, MCUCR
        lds     R29, RightEncoder

        sbrc    R30, ISC00      ; Was this an up or down event?
        rjmp    INT0_up

        subi    r29, lo8(-1)
        sbis    Right_Enc_B
        subi    R29, 2

        sbr     R30, 1<<ISC00   ; Switch Trigger to rising edge
        rjmp    INT0_done
INT0_up:
        subi    R29, 1
        sbis    Right_Enc_B
        subi    r29, lo8(-2)

        cbr     R30, 1<<ISC00   ; Switch Trigger to falling edge
INT0_done:
        sts     RightEncoder, R29
        out     MCUCR, R30
        out     SREG, R31
        pop     R29
        pop     R30
        pop     R31

        EndInterrupt
        _ENDFUNC

        _FUNCTION(SIG_INTERRUPT1)

SIG_INTERRUPT1:
        BeginInterrupt

        push    R31
        push    R30
        push    R29
        in      R31, SREG
        in      R30, MCUCR
        lds     R29, LeftEncoder

        sbrc    R30, ISC10      ; Was this an up or down event?
        rjmp    INT1_up

        subi    r29, lo8(-1)
        sbis    Left_Enc_B
        subi    R29, 2

        sbr     R30, 1<<ISC10   ; Switch Trigger to rising edge
        rjmp    INT1_done
INT1_up:
        subi    R29, 1
        sbis    Left_Enc_B
        subi    r29, lo8(-2)

        cbr     R30, 1<<ISC10   ; Switch Trigger to falling edge
INT1_done:
        sts     LeftEncoder, R29
        out     MCUCR, R30
        out     SREG, R31
        pop     R29
        pop     R30
        pop     R31

        EndInterrupt
        _ENDFUNC
        _END

