#define ENABLE_BIT_DEFINITIONS
#include "avrx-io.h"
#include "avrx-signal.h"
#include "avrx.h"
#include "hardware.h"
#include "Gyro_eeprom.h"

#ifdef _MONITOR_
  AVRX_GCC_TASK(Monitor, 16, 0);     // External task (AvrX Debug monitor)
  // AVRX_EXTERNTASK(Monitor);       // External task (monitor.c serialio.c)
  void InitSerialIO(unsigned char);
#endif

AVRX_EXTERNTASK(MotorTask);     // MotorTask.c
AVRX_EXTERNTASK(LcdTask);       // LcdTask.c
AVRX_EXTERNTASK(SensorTask);
//AVRX_EXTERNTASK(BehaviorTask);

AVRX_SIGINT(SIG_OVERFLOW0)  // System clock
{
    IntProlog();
    outp((TCNT0_INIT), TCNT0);
    AvrXTimerHandler();
    Epilog();
}
/*

  Main initialization and starting of AvrX tasks

*/
void main(void)
{
    AvrXSetKernelStack(0);

    // Initialize Hardware

    outp(MCUCR_INIT, MCUCR);    // See harware.h
    outp(TCNT0_INIT, TCNT0);
    outp(TCCR0_INIT, TCCR0);
    outp(TIMSK_INIT, TIMSK);
    outp(GIMSK_INIT, GIMSK);
    outp(TCCR1A_INIT, TCCR1A);
    outp(TCCR1B_INIT, TCCR1B);

    outp(PORTA_INIT, PORTA);
    outp(DDRA_INIT, DDRA);
    outp(PORTB_INIT, PORTB);
    outp(DDRB_INIT, DDRB);
    outp(PORTC_INIT, PORTC);
    outp(DDRC_INIT, DDRC);
    outp(PORTD_INIT, PORTD);
    outp(DDRD_INIT, DDRD);

    // Initialize tasks

    AvrXRunTask(TCB(MotorTask));
    AvrXRunTask(TCB(LcdTask));
    AvrXRunTask(TCB(SensorTask));
//    AvrXRunTask(TCB(BehaviorTask));

#ifdef _MONITOR_
    InitSerialIO(UBRR_INIT);
    AvrXRunTask(TCB(Monitor));
#endif

    AvrXSetSemaphore(&EEPromMutex);
    (void)ValidateEEPROM();
    // Jump to first task

    Epilog();
}
