/*
Gyro_eeprom.c

Copyright 2002 Larry Barello

Author: Larry Barello
        larry@barello.net
*/
#include "avrx.h"
#include "hardware.h"
#include "MotorTask.h"
#include "SensorTask.h"
#include "LcdTask.h"
#include "eeprom.h"
#include "Gyro_eeprom.h"

// Declare the programmed default modes
//
// If you maintain the "CheckSum" value, below, the system will
// boot after programming with the EEPROM valid.  Otherwise,
// make sure the values in "ResetEEPROM()" are the defaults you
// want when the EEPROM is initialized.

#define CHECKSUM  (unsigned) (0\
                  - (unsigned)KANGLE\
                  - (unsigned)KRATE\
                  - (unsigned)KIANGLE\
                  - (unsigned)KP\
                  - (unsigned)KD\
                  - (unsigned)KI\
                  - (unsigned)ACCELERATION\
                  - (unsigned)DRIVEOFFSET\
                  - (unsigned)ANGLEGAIN\
                  - (unsigned)GYROOFFSET\
                  )

unsigned short EEPROM
    eeKangle    = KANGLE,
    eeKrate     = KRATE,
    eeKiangle   = KIANGLE,
    eeKp        = KP,
    eeKd        = KD,
    eeKi        = KI,
    eeAcceleration=ACCELERATION,
    eeDriveOffset=DRIVEOFFSET,
    eeiAngleGain= ANGLEGAIN,
    eeiGyroOffset= GYROOFFSET,
    eeChkSum    = CHECKSUM;


unsigned short GenerateChecksum(void)
{
    unsigned short CheckSum = 0;

    CheckSum    -= eeprom_rw((unsigned)&eeKangle);
    CheckSum    -= eeprom_rw((unsigned)&eeKrate);
    CheckSum    -= eeprom_rw((unsigned)&eeKiangle);
    CheckSum    -= eeprom_rw((unsigned)&eeKp);
    CheckSum    -= eeprom_rw((unsigned)&eeKd);
    CheckSum    -= eeprom_rw((unsigned)&eeKi);
    CheckSum    -= eeprom_rw((unsigned)&eeAcceleration);
    CheckSum    -= eeprom_rw((unsigned)&eeDriveOffset);
    CheckSum    -= eeprom_rw((unsigned)&eeiAngleGain);
    CheckSum    -= eeprom_rw((unsigned)&eeiGyroOffset);
    return CheckSum;
}

void eeprom_ww(unsigned adr, unsigned data)
{
    eeprom_wb(adr, (char) data);
    eeprom_wb(adr+1, data >>8);
}

void WriteChecksum(void)
{
    eeprom_ww((unsigned)&eeChkSum, GenerateChecksum());
    (void)eeprom_rb(0x300);     // Hack: set eeprom pointer near end of memory
}

char ValidateEEPROM(void)
{
    char EEPROMCorrupt;

    if (GenerateChecksum() != eeprom_rw((unsigned)&eeChkSum))
        EEPROMCorrupt = TRUE;
    else
        EEPROMCorrupt = FALSE;

    (void)eeprom_rb(0x300);     // Hack: set eeprom pointer near end of memory

    return EEPROMCorrupt;
}

void ResetEEPROM(void)
{
    eeprom_ww((unsigned)&eeKangle, KANGLE);
    eeprom_ww((unsigned)&eeKrate,KRATE);
    eeprom_ww((unsigned)&eeKiangle, KIANGLE);
    eeprom_ww((unsigned)&eeKp,KP);
    eeprom_ww((unsigned)&eeKd, KD);
    eeprom_ww((unsigned)&eeKi, KI);
    eeprom_ww((unsigned)&eeAcceleration, ACCELERATION);
    eeprom_ww((unsigned)&eeDriveOffset, DRIVEOFFSET);
    eeprom_ww((unsigned)&eeiAngleGain, ANGLEGAIN);
    eeprom_ww((unsigned)&eeiGyroOffset, GYROOFFSET);
    eeprom_ww((unsigned)&eeChkSum, CHECKSUM);
}


