#ifndef CHEADER_HARDWARE
# define CHEADER_HARDWARE

/*
File:   HARDWARE.INC

Copyright 2000, 2001 Larry Barello

*/
#if !defined(M_PI)
#  define M_PI 3.1415926536
#endif
#define FALSE 0
#define TRUE 1

#define COUNTS_PER_REVOLUTION (24*60.5)

#define LEFT_WHEEL      4.3750      // Diameter: Each wheel is slightly different
#define RIGHT_WHEEL     4.3750
#define WHEEL_BASE      7.2000

#define LEFT_DIST_PER_TICK (M_PI * LEFT_WHEEL/COUNTS_PER_REVOLUTION)
#define RIGHT_DIST_PER_TICK (M_PI * RIGHT_WHEEL/COUNTS_PER_REVOLUTION)
#define LEFT_ONE_DEG    ((int)((2*M_PI*256)/((LEFT_DIST_TICK/WHEEL_BASE)*360))
#define RIGHT_ONE_DEG   ((int)((2*M_PI*256)/((RIGHT_DIST_TICK/WHEEL_BASE)*360))
#define DISTANCE(A)     (M_PI*(A)/COUNTS_PER_REVOLUTION)
#define ONE_DEGREE(A)   ((int)((2*M_PI*256)/(DISTANCE(A) * 360 / WHEEL_BASE)))

// Here is the CPU stuff

#define CPUCLK      8000000
#define BAUDRATE 19200
#define UBRR_INIT ((CPUCLK/16/BAUDRATE)-1)
//
// System Clock settings: CPU/256, 1.5khz tickrate
//
#define TICKRATE     125           // Picked to give an even divisor of CPU clock
#define MILLISECOND(A) ((A)*(long)TICKRATE/1000)
#define HERTZ(A)    (TICKRATE/(A))
#define TCCR0_INIT  (1<<CS02)
#define TCNT0_INIT  (256-(CPUCLK/256/TICKRATE))
//
// MCUCR = Int0/1 negative edge triggered, Sleep enabled
// TIMSK = Enable timer0 interrupt
// GIMSK = Enable int0/1 interrupts
//
#define MCUCR_INIT ((1<<SE) | (1<<ISC11) | (1<<ISC01))
#define TIMSK_INIT (1<<TOIE0)
#define GIMSK_INIT ((1<<INT1) | (1<<INT0))
//
// High Speed Counter settings:  CPU/1, 8 bit PWM
//
#define TCCR1A_INIT  ((1<<PWM10) | (1<<COM1A1) | (1<<COM1B1))
#define TCCR1B_INIT  (1<<CS10)
//
// Right Motor
// PD5 = OC1A
//
#define RIGHT_DRIVEH OCR1AH
#define RIGHT_DRIVEL OCR1AL
//
// Left Motor
// PD4 (OC1B)
//
#define LEFT_DRIVEH  OCR1BH
#define LEFT_DRIVEL  OCR1BL
//
// PORTB Bit Definitions
//
// PB 1-3 is the LCD/Switch data bus (4 bit)
// PB 5-7 are the LCD control lines
// PB 4 is the program LED (LCD/Switch select)
//
#define LED_ON()        cbi(PORTB, PB4)
#define LED_OFF()       sbi(PORTB, PB4)
#define LED_ENABLE()    sbi(DDRB, PB4)

#define DDRB_INIT       (1<<PB4)
#define PORTB_INIT      (DDRB_INIT)
//
// Port D Bit Definitions
//
// PD 0,1 used for Serial I/O
// PD 4,5 used for PWM output
// PD 2,3 used for interrupt driven encoder input.
//
#define Right_Enc_A     PIND, PD2   // INT0
#define Left_Enc_A      PIND, PD3   // INT1

#define DDRD_INIT     1<<PD5 | 1<<PD4 | 1<<PD1        // PWM and Txd are outputs
#define PORTD_INIT    ~DDRD_INIT    // Enable pullups on all inputs
//
// PORT A Bit Definitions
//
#define PORTA_INIT   0              // ADC port
#define DDRA_INIT    0

// ADC Channel Descriptions

#define GYRO    PA0
#define VBATT   PA2                 // * 20 / 93 = .1v
#define XTILT   PA1                 // Forward/Backwards, inverse
#define YTILT   PA3                 // Side tilt

//
// PORT C Bit Definitions
//
#define Right_Enc_B     PINC, PC5
#define RIGHT_FORWARD() cbi(PORTC, PC4); cbi(TCCR1A, COM1A0);
#define RIGHT_REVERSE() sbi(PORTC, PC4); sbi(TCCR1A, COM1A0);
#define LEFT_FORWARD()  cbi(PORTC, PC3); cbi(TCCR1A, COM1B0);
#define LEFT_REVERSE()  sbi(PORTC, PC3); sbi(TCCR1A, COM1B0);
#define Left_Enc_B      PINC, PC2

#define SW_UP           PC5     // PinD 6 & 7
#define SW_DOWN         PC4
#define SW_ITEM         PC6     // PinC 6 & 7 >> 2
#define SW_HALT         PC7
#define LCD_SWITCHES()  ((inp(PINC) & 0xC0)>>2)|(inp(PIND)&0xC0)

#define PORTC_INIT      (~DDRC_INIT)
#define DDRC_INIT       ((1<<PC4) + (1<<PC3))


#endif // CHEADER_HARDWARE

