/*
lcd_io4.s

Copyright 1998, 1999 Larry Barello

Author: Larry Barello
        larry@barello.net
*/
#define	__SFR_OFFSET 0       	/* GCC 3.2 needs this for direct low I/O access */
#include        <avrx-io.h>
#include        <avrx.inc>
#include        "lcd.inc"

        _MODULE(lcd)
;
; Hitachi LCD Control routines for industry standard LCD controllers.
; This code implements a flat (no stack usage) 4bit interface
;
; This code assumes that p1h is trashable
;
#define LCD_PARM p1l    // Register to pass/ret a byte parameter.
#define LCD_TMP p1h

        _CODESECTION
#if 0
#if defined(__IAR_SYSTEMS_ASM__)
LCD_DELAY      MACRO
        rjmp    $
        rjmp    $
        rjmp    $
        ENDM
#else
LCD_DELAY:      .macro  // number of delay cycles
        rjmp    .       //  4 mhz = 0
        rjmp    .       //  8 mhz = 1
        rjmp    .       // 16 mhz = 3
        .endm
#endif

#else

#define LCD_DELAY rcall lcd_fdelay

        _FUNCTION(lcd_fdelay)
lcd_fdelay:
        push    tmp0
        ldi     tmp0, 10
foo:
        dec     tmp0
        brne    foo
        pop     tmp0
        ret
        _ENDFUNC
#endif
;+
;----------------------------------------------
; void lcd_wr_data(uint8_t)
; void lcd_wr_command(uint8_t)
; uint8_t lcd_rd_data(void)
; uint8_t lcd_rd_command(void)
;
; RD/WR LCD Data and Instruction.
;
; PASSED:       T = Data/Instruction
;               LCD_PARM (R25)  = Write Data
; RETURN:       LCD_RMP  (R24)  = Read Data
; Registers:    LCD_TMP         = scratch
; Stack:        None
;-
        _FUNCTION(lcd_put_char)
        _PUBLIC(lcd_wr_data)
lcd_put_char:
lcd_wr_data:
        cbi     LCD_CTL, LCD_RW         ; RW = 0 = WRITE
        sbi     LCD_CTL, LCD_RS         ; RS = 1 = DATA
        rjmp    lcd_rw_rs
        _ENDFUNC

        _FUNCTION(lcd_rd_data)

lcd_rd_data:
        sbi     LCD_CTL, LCD_RW         ; RW = 1 = READ
        sbi     LCD_CTL, LCD_RS         ; RS = 1 = DATA
        rjmp    lcd_rw_rs
        _ENDFUNC

        _FUNCTION(lcd_wr_inst)

lcd_wr_inst:
        cbi     LCD_CTL, LCD_RW         ; RW = 0 = WRITE
        cbi     LCD_CTL, LCD_RS         ; RS = 0 = INSTRUCTION
        rjmp    lcd_rw_rs
        _ENDFUNC

        _FUNCTION(lcd_rd_inst)

lcd_rd_inst:
        sbi     LCD_CTL, LCD_RW         ; RW = 1 = READ
        cbi     LCD_CTL, LCD_RS         ; RS = 0 = INSTRUCTION
        rjmp    lcd_rw_rs
        _ENDFUNC

        _FUNCTION(lcd_rw_rs)

lcd_rw_rs:
        in      LCD_TMP, LCD_DATA_DDR
        cbr     LCD_TMP, LCD_DATA_BITS
        out     LCD_DATA_DDR, LCD_TMP   ; make DATA an input port.

        in      LCD_TMP, LCD_DATA_OUT
        cbr     LCD_TMP, LCD_DATA_BITS
        out     LCD_DATA_OUT, LCD_TMP   ; get rid of any spurious pullups
        in      LCD_TMP, LCD_CTL        ; R/S and R/W state in LCD_TMP
;
; Check busy flag
;
        sbi     LCD_CTL, LCD_RW         ; RW = 1 = READ
        cbi     LCD_CTL, LCD_RS         ; RS = 0 = INSTRUCTION
lcd_ck_bf:
        sbi     LCD_CTL, LCD_E
        LCD_DELAY
        set
        sbis    LCD_DATA_IN, LCD_BF     ; T = state of BF
        clt
        cbi     LCD_CTL, LCD_E
        LCD_DELAY
        sbi     LCD_CTL, LCD_E
        LCD_DELAY
        cbi     LCD_CTL, LCD_E
        LCD_DELAY
        brts    lcd_ck_bf               ; Loop while BF = 1
;
; Restore R/S and R/W
;
        sbrc    LCD_TMP, LCD_RS
        sbi     LCD_CTL, LCD_RS         ; RS = 1 = DATA

        sbrc    LCD_TMP, LCD_RW
        rjmp    lcd_rd
lcd_wr:
        cbi     LCD_CTL, LCD_RW         ; RW = 0 = WRITE
        in      LCD_TMP, LCD_DATA_DDR   ; make DATA an output port.
        sbr     LCD_TMP, LCD_DATA_BITS
        out     LCD_DATA_DDR, LCD_TMP
;
; 4 bit I/O Write operation (LCD_CTL restored)
;
        sbi     LCD_CTL, LCD_E          ; Raise E (do this before capture state
                                        ; of LCD_DATA_OUT in case same port used)
        in      LCD_TMP, LCD_DATA_OUT   ; Save a copy of data port
#if     (LCD_DATA_BITS==LCD_LOW_NIBBLE)
        swap    LCD_PARM                ; Put out High nibble first
#endif
        cbr     LCD_TMP, LCD_DATA_BITS
        eor     LCD_TMP, LCD_PARM       ; Or in the data (high and low nibbles)
        cbr     LCD_PARM, LCD_DATA_BITS ; cancel data in PARM
        eor     LCD_TMP, LCD_PARM       ; Cancel out other nibble
        out     LCD_DATA_OUT, LCD_TMP

        LCD_DELAY                       ; Need to delay 500 ns (total)
        cbi     LCD_CTL, LCD_E          ; lower E

        LCD_DELAY                       ; Need to delay 500 ns

        swap    LCD_PARM                ; Put out the other nibble
        cbr     LCD_TMP, LCD_DATA_BITS
        or      LCD_TMP, LCD_PARM       ; Low nibble

        sbi     LCD_CTL, LCD_E
        out     LCD_DATA_OUT, LCD_TMP
        LCD_DELAY                       ; Need to delay 500 ns (total)
        cbi     LCD_CTL, LCD_E

        ret
;
; 4 bit I/O Read operation (LCD_CTL restored)
; High nibble read first, then low.
;
lcd_rd:
        sbi     LCD_CTL, LCD_E          ; Raise E
        LCD_DELAY
        in      LCD_PARM, LCD_DATA_IN
        cbi     LCD_CTL, LCD_E          ; Lower E

        cbr     LCD_PARM, (~LCD_DATA_BITS)&0xFF

#if     (LCD_DATA_BITS==LCD_LOW_NIBBLE)
        swap    LCD_PARM                ; Put nibble in proper place
#endif
        LCD_DELAY

        sbi     LCD_CTL, LCD_E
        LCD_DELAY
        in      LCD_TMP, LCD_DATA_IN
        cbi     LCD_CTL, LCD_E

        cbr     LCD_TMP, (~LCD_DATA_BITS)&0xFF

#if     (LCD_DATA_BITS==LCD_HI_NIBBLE)
        swap    LCD_TMP                ; Put out High nibble first
#endif
        or      LCD_PARM, LCD_TMP      ; Return data in R24 (C calling convention)
        ret
        _ENDFUNC
;
;--------------------------------------------------
;
; lcd_4_init
;
; Switches reset LCD to 4 bit I/F mode.
;
; PASSED:       Nothing
; RETURNS:
; USES:         LCD_PARM (R24), LCD_TMP, T
; NOTES:        Initial read of Busy Flag and write to controller is 8 bit,
;               susequent i/o is 4 bit (dual r/w cycles).  Need to do a second
;               function set to get all bits to the lcd display.
;
        _FUNCTION(lcd_4_init)

lcd_4_init:
        ldi     LCD_PARM, LCD_CTL_DDR_INIT
        out     LCD_CTL_DDR, LCD_PARM

        sbi     LCD_CTL, LCD_RW         ; RW = 1 = READ
        cbi     LCD_CTL, LCD_RS         ; RS = 0 = INSTRUCTION
lcd_init_ck_bf:
        cbi     LCD_CTL, LCD_E
        LCD_DELAY
        sbi     LCD_CTL, LCD_E
        LCD_DELAY
        sbic    LCD_DATA_IN, LCD_BF     ; msb of nibble
        rjmp    lcd_init_ck_bf          ; Wait for busy flag to clear

        cbi     LCD_CTL, LCD_E          ; Switch to output mode
        cbi     LCD_CTL, LCD_RW         ; RW = 0 = READ
        in      LCD_TMP, LCD_DATA_DDR
        sbr     LCD_TMP, LCD_DATA_BITS
        out     LCD_DATA_DDR, LCD_TMP

        LCD_DELAY
        sbi     LCD_CTL, LCD_E
        nop                             ; 1 cycle for Pin to reflect Port.
        in      LCD_TMP, LCD_DATA_IN    ; Set mode to 4 bit I/O
        cbr     LCD_TMP, LCD_DATA_BITS
#if     (LCD_DATA_BITS==0x0F)
        ori     LCD_TMP, (LCD_FUNCTION_SET)>>4
#else
        ori     LCD_TMP, LCD_FUNCTION_SET
#endif
        out     LCD_DATA_OUT, LCD_TMP
        LCD_DELAY
        cbi     LCD_CTL, LCD_E

        ldi     LCD_PARM, LCD_FUNCTION_SET
        rjmp    lcd_wr_inst             ; now set function via 4 bit I/O
        _ENDFUNC
        _END

