/*
lcd formatted i/o and access routines

Copyright 1998, 1999 Larry Barello

Author: Larry Barello
        larry@barello.net
*/
#include "avrX.inc"
#include "lcd.inc"
#include "avrx-ctoasm.inc"

        _MODULE(lcdio)
        _DATASECTION

        _EXTERN(AvrXSetSemaphore)
        _EXTERN(AvrXWaitSemaphore)
        _EXTERN(lcd_wr_data)
        _EXTERN(lcd_wr_inst)
        _EXTERN(lcd_rd_inst)
;+
; Change History
;
; 4-13-00       Lcd_EraseToEndOfLine
;               What was I thinking?  The rjmp after the call
;               to print out the space was "brne"?  Duh?  There
;               is no return value from Lcd_WriteCommand!
;-
        _GLOBAL(lcdmutex, MtxSz)
;+
;------------------------------------------------------------------
; lcd_Get/ReleaseDisplay
;
; Uses lcdmutex to control access to the LCD driver
;
; PASSED:       Nothing
; RETURNS:      Nothing
; USES:         Nothing
; NOTES:        Calling process may block waiting for access
;               to the LCD display.  If you call this twice
;               you will DEADLOCK
;-
        _CODESECTION
        
        _FUNCTION(lcd_GetDisplay)

lcd_GetDisplay:
        ldi     p1l, lo8(lcdmutex)
        ldi     p1h, hi8(lcdmutex)
        rjmp    AvrXWaitSemaphore
        _ENDFUNC

        _FUNCTION(lcd_ReleaseDisplay)
lcd_ReleaseDisplay:
        ldi     p1l, lo8(lcdmutex)
        ldi     p1h, hi8(lcdmutex)
        rjmp    AvrXSetSemaphore
        _ENDFUNC

;-----------------------------------------
;void lcd_PutHex(uint16_t, uint8_t)
; 
; Passed:       p1wh:p1wl = Value, p2b = # digits
; RETURN:       Nothing
; USES:         X, R22
; STACK:        2
;
        _FUNCTION(lcd_PutHex)

lcd_PutHex:
        mov     tmp1, p1l
        mov     tmp2, p1h
        ldi     tmp0, 4          ; Maximum number of digits.
lcdph1:
        lsl     tmp1
        rol     tmp2
        rol     p1l
        lsl     tmp1
        rol     tmp2
        rol     p1l
        lsl     tmp1
        rol     tmp2
        rol     p1l
        lsl     tmp1
        rol     tmp2
        rol     p1l

        cp      p2l, tmp0
        brlt    lcdph3

        cbr     p1l, 0xF0
        subi    p1l, -'0'          ; Addi 0
        cpi     p1l, ('9' + 1)     ; if > 9
        brlt    lcdph2             ; subi 9 and add A
        subi    p1l, ('9' + 1 - 'A')
lcdph2:
        rcall   lcd_wr_data     ; Uses p1{h,l}
lcdph3:  
        dec     tmp0
        brne    lcdph1

        ret
        _ENDFUNC
        
;-----------------------------------------
; void lcd_PutHexWord(uint16_t)
; 
; Passed: p1h:p1h = word
;
        _FUNCTION(lcd_PutHexWord)
lcd_PutHexWord:
        ldi     p2l, 4
        rjmp    lcd_PutHex
        _ENDFUNC

;-----------------------------------------
; void lcd_PutHexByte(uint8_t)
; 
; Passed: R25 = Byte
;
        _FUNCTION(lcd_PutHexByte)

lcd_PutHexByte:
        ldi     p2l, 2
        rjmp    lcd_PutHex
        _ENDFUNC

        _FUNCTION(lcd_PutHexChar)

lcd_PutHexChar:
        ldi     p2l, 1
        rjmp    lcd_PutHex
        _ENDFUNC
        

;
;--------------------------------------------------
;
; void lcd_putstr(pFlash)
;
; Puts out an ascii string to the LCD display
;
; PASSED:       Z = pointer to string in CODE
; RETURNS:
; USES:         LCD_PARM, R0
; CALLS:        lcd_wr_data
; NOTES:        
;
        _FUNCTION(lcd_putstr)

lcd_putstr:
        mov     Zl, p1l
        mov     Zh, p1h
lcdps1:
        lpm
        mov     p1l, R0
        tst     p1l
        breq    ps_x
        rcall   lcd_wr_data
        adiw    Zl, 1
        rjmp    lcdps1
ps_x:
        ret
        _ENDFUNC

;+
; void lcd_PrintMessageAt(pFlash, Cursor)
;
; Print a FLASH string at a location set by LCD_PARM
; Erase all charactors after the string, to the end of line
;
; Passed:       p1wl:p1wl = Pointer to Flash String
;               p2b = LCD cursor location
; Returns:      nothing
; Uses:         Z
;-
        _FUNCTION(lcd_PrintMessageAt)

lcd_PrintMessageAt:
        push    p1l
        push    p1h
        ldi     p1l, LCD_SET_DDRAM
        or      p1l, p2l
        rcall   lcd_wr_inst
        pop     p1h
        pop     p1l
        rcall   lcd_putstr
        rcall   lcd_EraseToEndOfLine
        ret
        _ENDFUNC
;+
; void lcd_EraseToEndOfLine(void)
;
; Erases the LCD from the current cursor position to
; the end of the line.
;
; Passed:       Nothing
; Returns:      Nothing
; Uses:         LCD_PARAM
; NOTE:         Writes " " before checking so it
;               will clear from start of line.
;-
        _FUNCTION(lcd_EraseToEndOfLine)

lcd_EraseToEndOfLine:
        ldi     p1l, 0x20
        rcall   lcd_wr_data
        
        rcall   lcd_rd_inst     ; Clear to end of line.
        cbr     r1l, 0x80
        cpi     r1l, LCD_LINE1
        breq    EraseDone
        cpi     r1l, LCD_LINE2
        breq    EraseDone
        cpi     r1l, LCD_LINE3
        breq    EraseDone
        cpi     r1l, LCD_LINE4
        brne    lcd_EraseToEndOfLine
        
EraseDone:
        ret
        _ENDFUNC
        _END
        
